﻿Module Utils_ListViewColumnSort


    ' -------------------------------------------------------------------- The column currently used for sorting.
    Private m_SortingColumn As ColumnHeader = Nothing

    Friend Sub Sort_ListViewColumn(ByVal lv As ListView, Optional ByVal column As Int32 = -1)
        '
        ' -------------------------------------------------------------------- not sortable columns
        'If e.Column <= 0 Or e.Column >= 19 Then Exit Sub
        '
        ' -------------------------------------------------------------------- Get the new sorting column.
        Dim new_sorting_column As ColumnHeader = lv.Columns(column)
        ' -------------------------------------------------------------------- Figure out the new sorting order.
        Dim sort_order As System.Windows.Forms.SortOrder
        If m_SortingColumn Is Nothing Then
            ' ---------------------------------------------------------------- New column. Sort ascending.
            sort_order = SortOrder.Ascending
        Else
            ' ---------------------------------------------------------------- See if this is the same column.
            If new_sorting_column.Equals(m_SortingColumn) Then
                ' Same column. Switch the sort order.
                If m_SortingColumn.Text.StartsWith("> ") Then
                    sort_order = SortOrder.Descending
                Else
                    sort_order = SortOrder.Ascending
                End If
            Else
                ' ------------------------------------------------------------ New column. Sort ascending.
                sort_order = SortOrder.Ascending
            End If

            ' ---------------------------------------------------------------- Remove the old sort indicator.
            m_SortingColumn.Text = Strings.Mid(m_SortingColumn.Text, 3)
        End If

        ' -------------------------------------------------------------------- Display the new sort order.
        If sort_order = SortOrder.Ascending Then
            new_sorting_column.Text = "> " & new_sorting_column.Text
        Else
            new_sorting_column.Text = "< " & new_sorting_column.Text
        End If

        ' -------------------------------------------------------------------- Create a comparer.
        lv.ListViewItemSorter = New ListViewComparer(column, sort_order)

        ' -------------------------------------------------------------------- Sort.
        lv.Sort()
        lv.ListViewItemSorter = Nothing

        ' -------------------------------------------------------------------- Resize
        'If m_SortingColumn IsNot Nothing Then
        '    If m_SortingColumn.Index >= 0 Then
        '        lv.Columns(m_SortingColumn.Index).AutoResize(ColumnHeaderAutoResizeStyle.HeaderSize)
        '    End If
        'End If
        'lv.Columns(new_sorting_column.Index).AutoResize(ColumnHeaderAutoResizeStyle.HeaderSize)

        ' -------------------------------------------------------------------- Update old
        m_SortingColumn = new_sorting_column
    End Sub

    Friend Sub Sort_ListView_Stop()
        If m_SortingColumn Is Nothing Then Exit Sub
        If m_SortingColumn.Text.StartsWith("> ") Or m_SortingColumn.Text.StartsWith("< ") Then
            m_SortingColumn.Text = Strings.Mid(m_SortingColumn.Text, 3)
            m_SortingColumn = Nothing
        End If
    End Sub

    Class ListViewComparer
        Implements IComparer

        Private m_ColumnNumber As Integer
        Private m_SortOrder As SortOrder

        Public Sub New(ByVal column_number As Integer, ByVal sort_order As SortOrder)
            m_ColumnNumber = column_number
            m_SortOrder = sort_order
        End Sub

        Public Function Compare(ByVal x As Object, ByVal y As Object) As Integer Implements System.Collections.IComparer.Compare
            Dim item_x As ListViewItem = DirectCast(x, ListViewItem)
            Dim item_y As ListViewItem = DirectCast(y, ListViewItem)
            ' --------------------------------------------------------- Get the sub-item values
            Dim string_x As String
            If item_x.SubItems.Count <= m_ColumnNumber Then
                string_x = ""
            Else
                string_x = item_x.SubItems(m_ColumnNumber).Text
            End If
            Dim string_y As String
            If item_y.SubItems.Count <= m_ColumnNumber Then
                string_y = ""
            Else
                string_y = item_y.SubItems(m_ColumnNumber).Text
            End If

            ' --------------------------------------------------------- specific for FastFTP folders
            'If item_x.SubItems.Count >= 3 Then
            '    If item_x.SubItems(2).Text = "dir" Then string_x = "_" & string_x
            'End If
            'If item_y.SubItems.Count >= 3 Then
            '    If item_y.SubItems(3).Text = "dir"  Then string_y = "_" & string_y
            'End If

            ' --------------------------------------------------------- Compare them
            If m_SortOrder = SortOrder.Ascending Then
                If IsNumeric(string_x) And IsNumeric(string_y) Then
                    Return Val(string_x).CompareTo(Val(string_y))
                ElseIf IsDate(string_x) And IsDate(string_y) Then
                    Return DateTime.Parse(string_x).CompareTo(DateTime.Parse(string_y))
                Else
                    Return String.Compare(string_x, string_y)
                End If
            Else
                If IsNumeric(string_x) And IsNumeric(string_y) Then
                    Return Val(string_y).CompareTo(Val(string_x))
                ElseIf IsDate(string_x) And IsDate(string_y) Then
                    Return DateTime.Parse(string_y).CompareTo(DateTime.Parse(string_x))
                Else
                    Return String.Compare(string_y, string_x)
                End If
            End If
        End Function
    End Class

End Module
